#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Dec 11 17:27:01 2020

@author: evianne
"""


#!/usr/bin/env python3

# standard library modules
import sys, errno, re, json, ssl
from urllib import request
from urllib.error import HTTPError
from time import sleep
from openpyxl import load_workbook
workbook = load_workbook(filename="UniprotID.xlsx")
sheet = workbook.active
UniprotID = []

for col in sheet['A']:
    UniprotID.append(col.value)
    
Uniprot = UniprotID[1:480]

sys.stdout = open('human_enz_interpro.tsv', 'w')

for x in range(1635):
    print(x, file=sys.stderr, end=" ")
    BASE_URL = "http://www.ebi.ac.uk/interpro/api/entry/all/protein/reviewed/"+Uniprot[x]+"/"
    
    def parse_items(items):
      if type(items)==list:
        return ",".join(items)
      return ""
    def parse_member_databases(dbs):
      if type(dbs)==dict:
        return ";".join([f"{db}:{','.join(dbs[db])}" for db in dbs.keys()])
      return ""
    def parse_go_terms(gos):
      if type(gos)==list:
        return ",".join([go["identifier"] for go in gos])
      return ""
    def parse_locations(locations):
      if type(locations)==list:
        return ",".join(
          [",".join([f"{fragment['start']}..{fragment['end']}" 
                    for fragment in location["fragments"]
                    ])
          for location in locations
          ])
      return ""
    def parse_group_column(values, selector):
      return ",".join([parse_column(value, selector) for value in values])
    
    def parse_column(value, selector):
      if value is None:
        return ""
      elif "member_databases" in selector:
        return parse_member_databases(value)
      elif "go_terms" in selector: 
        return parse_go_terms(value)
      elif "children" in selector: 
        return parse_items(value)
      elif "locations" in selector:
        return parse_locations(value)
      return str(value)
    
    def output_list():
      #disable SSL verification to avoid config issues
      context = ssl._create_unverified_context()
    
      next = BASE_URL
      last_page = False
    
      
      attempts = 0
      while next:
        try:
          req = request.Request(next, headers={"Accept": "application/json"})
          res = request.urlopen(req, context=context)
          # If the API times out due a long running query
          if res.status == 408:
            # wait just over a minute
            sleep(61)
            # then continue this loop with the same URL
            continue
          elif res.status == 204:
            #no data so leave loop
            break
          payload = json.loads(res.read().decode())
          next = payload["next"]
          attempts = 0
          if not next:
            last_page = True
        except HTTPError as e:
          if e.code == 408:
            sleep(61)
            continue
          else:
            # If there is a different HTTP error, it wil re-try 3 times before failing
            if attempts < 3:
              attempts += 1
              sleep(61)
              continue
            else:
              sys.stderr.write("LAST URL: " + next)
              raise e
    
        for i, item in enumerate(payload["results"]):
          sys.stdout.write(parse_column(item["metadata"]["accession"], 'metadata.accession') + "\t")
          sys.stdout.write(parse_column(item["metadata"]["source_database"], 'metadata.source_database') + "\t")
          sys.stdout.write(parse_column(item["metadata"]["name"], 'metadata.name') + "\t")
          sys.stdout.write(parse_column(item["metadata"]["member_databases"], 'metadata.member_databases') + "\t")
          if ("IPR" in (item["metadata"]["accession"])) and (item["metadata"]["go_terms"]=="null") :
              for j in len(["metadata"]["go_terms"]):
                  sys.stdout.write(parse_column(item["metadata"]["go_terms"][j]["identifier"], 'metadata.go_terms.identifier') + "\t")
          else: sys.stdout.write(parse_column(item["metadata"]["go_terms"], 'metadata.go_terms') + "\t")
          sys.stdout.write(parse_column(item["proteins"][0]["accession"], 'proteins.acession') + "\t")
          if len(item["proteins"][0]["entry_protein_locations"])>1 :
              sys.stdout.write(parse_column(item["proteins"][0]["entry_protein_locations"][0]["fragments"][0]["start"], 'proteins.fragments.start') + "\t")
              sys.stdout.write(parse_column(item["proteins"][0]["entry_protein_locations"][0]["fragments"][0]["end"], 'proteins.fragments.end') + "\t")
              sys.stdout.write("\n")
              for m in range(1,len(item["proteins"][0]["entry_protein_locations"])):
                  sys.stdout.write(parse_column(item["metadata"]["accession"], 'metadata.accession') + "\t")
                  sys.stdout.write(parse_column(item["metadata"]["source_database"], 'metadata.source_database') + "\t")
                  sys.stdout.write(parse_column(item["metadata"]["name"], 'metadata.name') + "\t")
                  sys.stdout.write(parse_column(item["metadata"]["member_databases"], 'metadata.member_databases') + "\t")
                  if ("IPR" in (item["metadata"]["accession"])) and (item["metadata"]["go_terms"]=="null") :
                      for j in len(["metadata"]["go_terms"]):
                          sys.stdout.write(parse_column(item["metadata"]["go_terms"][j]["identifier"], 'metadata.go_terms.identifier') + "\t")
                  else: sys.stdout.write(parse_column(item["metadata"]["go_terms"], 'metadata.go_terms') + "\t")
                  sys.stdout.write(parse_column(item["proteins"][0]["accession"], 'proteins.acession') + "\t")
                  sys.stdout.write(parse_column(item["proteins"][0]["entry_protein_locations"][m]["fragments"][0]["start"], 'proteins.fragments.start') + "\t")
                  sys.stdout.write(parse_column(item["proteins"][0]["entry_protein_locations"][m]["fragments"][0]["end"], 'proteins.fragments.end') + "\t")
                  sys.stdout.write("\n")
          else: 
              sys.stdout.write(parse_column(item["proteins"][0]["entry_protein_locations"][0]["fragments"][0]["start"], 'proteins.fragments.start') + "\t")
              sys.stdout.write(parse_column(item["proteins"][0]["entry_protein_locations"][0]["fragments"][0]["end"], 'proteins.fragments.end') + "\t")
              sys.stdout.write("\n")
          
          # Don't overload the server, give it time before asking for more
        if next:
          sleep(1)
    
    if __name__ == "__main__":
      output_list()
      
